package com.hand.hls.dp.util;

import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Objects;

/**
 * @author <a href="mailto:zhangnet14@gmail.com">Spencer Chang</a>
 * @date 2020/4/18 16:37
 * @since
 */
public class SqlUtils {
    public static boolean checkSql(String sql, String... conditions) {
        Objects.requireNonNull(conditions, "conditions 条件不能为空");
        if (conditions.length < 1) {
            return false;
        }

        for (String condition : conditions) {
            if (StringUtils.contains(sql, condition)) {
                return true;
            }
        }
        return false;
    }

    public static boolean checkSql4WhereOneToOne(String sql) {
        int lastIndex = StringUtils.lastIndexOfIgnoreCase(sql, SqlConstantUtils.SQL_WHERE);
        String condition = StringUtils.substringAfterLast(sql, StringUtils.substring(sql, lastIndex, lastIndex + 5));
        String[] conditions = StringUtils.split(StringUtils.trimToEmpty(condition), SqlConstantUtils.SQL_WHERE_ADD);
        if (ArrayUtils.isNullOrEmpty(conditions)) {
            conditions = StringUtils.split(condition, SqlConstantUtils.SQL_WHERE_ADD_UPPER);
        }
        for (String c : conditions) {
            c = StringUtils.deleteWhitespace(StringUtils.normalizeSpace(c));
            String[] tmpCons = StringUtils.split(c, SqlConstantUtils.SQL_WHERE_1T1_EQ);
            if (!ArrayUtils.isNullOrEmpty(tmpCons)) {
                List<String> cons = ArrayUtils.nonNullElementsIn(tmpCons);
                cons.remove(SqlConstantUtils.SQL_WHERE_1T1_EQ);
                for (int i = 0; i < cons.size(); i++) {
                    if (i == 1) {
                        String startCon = cons.get(i - 1);
                        String endCon = cons.get(i);
                        if (StringUtils.equals(startCon, endCon)) {
                            return true;
                        }
                    }
                }
            }
        }
        return false;
    }

    public static boolean checkSqlIgnoreCase(String sql, String... conditions) {
        Objects.requireNonNull(conditions, "conditions 条件不能为空");
        if (conditions.length < 1) {
            throw new IllegalArgumentException("conditions 条件不能为空");
        }

        for (String condition : conditions) {
            if (StringUtils.containsIgnoreCase(sql, condition)) {
                if (StringUtils.equals(condition, SqlConstantUtils.SQL_STAR)){
                    String tmpSql = sql;
                    int count = StringUtils.countMatches(tmpSql, SqlConstantUtils.SQL_STAR);
                    for (int i = 0; i < count; i++) {
                        tmpSql = StringUtils.trim(StringUtils.substring(tmpSql, SqlConstantUtils.SQL_STAR));
                        if（StringUtils.startsWithIgnoreCase(tmpSql, SqlConstantUtils.SQL_FROM) || StringUtils.startsWithIgnoreCase(tmpSql, SqlConstantUtils.SQL_COMMA)）{
                            return true;
                        }
                    }
                } else {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean checkSqlFilterTableIgnoreCase(String sql, String... conditions) {
        Objects.requireNonNull(conditions, "conditions 条件不能为空");
        if (conditions.length < 1) {
            throw new IllegalArgumentException("conditions 条件不能为空");
        }

        for (String condition : conditions) {
            if (StringUtils.containsIgnoreCase(sql, condition)) {
                return true;
            }
        }
        return false;
    }

    public static boolean checkSqlStartWithIgnoreCase(String sql, String... conditions) {
        Objects.requireNonNull(conditions, "conditions 条件不能为空");
        if (conditions.length < 1) {
            throw new IllegalArgumentException("conditions 条件不能为空");
        }
        for (String condition : conditions) {
            if (!StringUtils.startsWith(sql, condition)) {
                return true;
            }
        }
        return false;
    }

    public static boolean checkSqlStartWith4DdlIgnoreCase(String sql, String... conditions) {
        Objects.requireNonNull(conditions, "conditions 条件不能为空");
        if (conditions.length < 1) {
            throw new IllegalArgumentException("conditions 条件不能为空");
        }
        if (StringUtils.startsWith(sql, SqlConstantUtils.DDL_CREATE_TABLE)) {
            return false;
        } else if (StringUtils.startsWith(sql, SqlConstantUtils.DDL_ALTER_TABLE)) {
            return false;
        } else if (StringUtils.startsWith(sql, SqlConstantUtils.DDL_COMMENT_TABLE)) {
            return false;
        } else if (StringUtils.startsWith(sql, SqlConstantUtils.DDL_COMMENT_ON_COLUMN)) {
            return false;
        } else if (StringUtils.startsWith(sql, SqlConstantUtils.DDL_CREATE_INDEX)) {
            return false;
        } else if (StringUtils.startsWith(sql, SqlConstantUtils.DDL_CREATE_SEQUENCE)) {
            return false;
        }
        for (String condition : conditions) {
            if (StringUtils.startsWith(sql, condition)) {
                return true;
            }
        }
        return false;
    }

    public static boolean checkSqlEndsWithIgnoreCase(String sql, String... conditions) {
        Objects.requireNonNull(conditions, "conditions 条件不能为空");
        if (conditions.length < 1) {
            throw new IllegalArgumentException("conditions 条件不能为空");
        }

        for (String condition : conditions) {
            if (StringUtils.endsWithIgnoreCase(sql, condition)) {
                return true;
            }
        }
        return false;
    }
}
